@@ -436,11 +436,19 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
     }
     entry = lookup_ip(itf,ip);
     assert(!vcc || vcc->entry);
+    /*
+     * If the entry on which we received the update isn't dangling but it
+     * doesn't correspond to the one with the address, ...
+     */
     if (entry && vcc && vcc->entry->itf && entry != vcc->entry) {
 	diag(COMPONENT,DIAG_DEBUG,"collision on %d.%d.%d.%d",ipp[0],ipp[1],
 	  ipp[2],ipp[3]);
 	return;
     }
+    /*
+     * If the entry on which we received the update is dangling and we found
+     * an entry that already describes that IP address, ...
+     */
     if (vcc && entry && !vcc->entry->itf) {
 	if (!entry->svc) {
 	    diag(COMPONENT,DIAG_ERROR,"attempt to overwrite PVC for IP "
@@ -454,7 +462,12 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	Q_INSERT_HEAD(entry->vccs,vcc);
 	vcc = NULL;
     }
-    if (entry || (vcc && (entry = vcc->entry))) {
+    /*
+     * If we either found an entry or if the update was received via InARP
+     * (so we can use the entry that already belongs to the VCC), ...
+     */
+    if (entry || vcc) {
+	if (!entry) entry = vcc->entry;
 	if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
     }
     else {
@@ -469,6 +482,13 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
     entry->ip = ip;
     if (!entry->itf) {
 	entry->itf = itf;
+	/* @@@
+	 * Need to fix this is in case we allow entries without a valid IP
+	 * address but with a pre-set QOS, e.g. a VC on a given PVC with an
+	 * unknown remote end.
+	 */
+	entry->qos = entry->itf->qos;
+	adjust_qos(entry->itf,&entry->qos,0);
 	Q_INSERT_HEAD(itf->table,entry);
     }
     if (entry->itf != itf)
@@ -476,8 +496,6 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	  "(%d -> %d)",entry->itf->number,itf->number);
     if (!entry->addr) entry->addr = alloc(sizeof(*addr));
     *entry->addr = *addr;
-    entry->qos = entry->itf->qos;
-    adjust_qos(entry->itf,&entry->qos,0);
     for (walk = entry->vccs; walk; walk = walk->next)
 	if (!walk->connecting)
 	    if (set_ip(walk->fd,ip) < 0)
@@ -542,6 +560,7 @@ void need_ip(int itf_num,unsigned long ip)
 	entry->ip = ip;
 	entry->itf = itf;
 	Q_INSERT_HEAD(itf->table,entry);
+	entry->qos = itf->qos;
     }
     revalidate(entry);
 }
