@@ -16,10 +16,10 @@
 #define T_ATMA		34
 #endif
 #ifndef ATMA_AESA
-#define ATM_AESA	0
+#define ATMA_AESA	0
 #endif
-#ifndef ATM_E164
-#define ATM_E164	1
+#ifndef ATMA_E164
+#define ATMA_E164	1
 #endif
 
 /* Return codes for text2atm and atm2text */
