@@ -5,6 +5,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <linux/atm.h>
@@ -64,6 +65,7 @@ int main(int argc,char **argv)
 		    if (argc < 3 || argc > 4) usage(name);
 		    if (argc == 4) itf = argv[2];
 		    cmd = argv[1][1] == 'a' ? ATM_ADDADDR : ATM_DELADDR;
+		    memset(&addr,0,sizeof(addr));
 		    if (text2atm(argv[argc-1],(struct sockaddr *) addr,
 		      sizeof(*addr),T2A_SVC | T2A_NAME) < 0) usage(name);
 		    break;
