@@ -26,7 +26,7 @@ int q_dump = 0;
 static int debug = 0;
 
 
-void q_report(int severity,const char *msg,...)
+void PREFIX(report)(int severity,const char *msg,...)
 {
     va_list ap;
 
@@ -84,10 +84,10 @@ static void q_put(unsigned char *table,int pos,int size,unsigned long value)
 {
     int end;
  
-    q_report(Q_DEBUG,"put %d %d %ld",pos,size,value);
+    PREFIX(report)(Q_DEBUG,"put %d %d %ld",pos,size,value);
     end = pos+size;
     if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
-	q_report(Q_FATAL,"unsupported alignment (put %d,%d)",pos,size);
+	PREFIX(report)(Q_FATAL,"unsupported alignment (put %d,%d)",pos,size);
     if (size <= 8) {
 	unsigned char *here;
 	int shift;
@@ -112,10 +112,10 @@ static unsigned long q_get(unsigned char *table,int pos,int size)
     unsigned long value;
     int end;
  
-    q_report(Q_DEBUG,"get %d %d ...",pos,size);
+    PREFIX(report)(Q_DEBUG,"get %d %d ...",pos,size);
     end = pos+size;
     if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
-	q_report(Q_FATAL,"unsupported alignment (get %d,%d)",pos,size);
+	PREFIX(report)(Q_FATAL,"unsupported alignment (get %d,%d)",pos,size);
     if (size <= 8) value = (table[pos >> 3] >> (pos & 7)) & ((1 << size)-1);
     else {
 	table += pos >> 3;
@@ -125,7 +125,7 @@ static unsigned long q_get(unsigned char *table,int pos,int size)
 	    size -= 8;
         }
     }
-    q_report(Q_DEBUG,"  %ld",value);
+    PREFIX(report)(Q_DEBUG,"  %ld",value);
     return value;
 }
 
@@ -178,7 +178,7 @@ static int q_init(Q_DSC *dsc)
 	return -1;
     }
     memset(dsc->field_present,0,bytes);
-    bytes = (Q_GROUPS+7) >> 3;
+    bytes = (Q_GROUPS+(sizeof(unsigned long)*8-1)) >> 3;
     dsc->group_present = malloc(bytes);
     if (!dsc->group_present) {
 	perror("out of memory");
@@ -207,7 +207,7 @@ static void use_group(Q_DSC *dsc,int group)
     int *scan;
 
     while (group != -1) {
-	q_set(dsc->group_present,group);
+	q_set((unsigned char *) dsc->group_present,group);
 	for (scan = groups[group].required; scan && *scan != -1; scan++)
 	    q_set(dsc->required,*scan);
 	group = groups[group].parent;
@@ -220,17 +220,17 @@ void q_assign(Q_DSC *dsc,int field,unsigned long value)
     int *walk;
 
     if (field < 0 || field >= Q_FIELDS)
-	q_report(Q_FATAL,"invalid field value (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid field value (%d)",field);
     if (!fields[field].values) {
 	if (q_test(dsc->field_present,field)) /* probably an error ... */
-	    q_report(Q_ERROR,"changing field %d",field);
+	    PREFIX(report)(Q_ERROR,"changing field %d",field);
 	q_set(dsc->field_present,field);
 	q_put(dsc->data,fields[field].pos,fields[field].size,value);
 	use_group(dsc,fields[field].parent);
     }
     else {
 	if (q_test(dsc->field_present,field))
-	    q_report(Q_FATAL,"can't change field %d",field);
+	    PREFIX(report)(Q_FATAL,"can't change field %d",field);
 	q_set(dsc->field_present,field);
 	q_put(dsc->data,fields[field].pos,fields[field].size,value);
 	for (walk = fields[field].values; walk[1] != -1; walk += 2)
@@ -238,7 +238,7 @@ void q_assign(Q_DSC *dsc,int field,unsigned long value)
 		use_group(dsc,walk[1]);
 		return;
 	    }
-	q_report(Q_ERROR,"invalid value (%d in field %d)",value,field);
+	PREFIX(report)(Q_ERROR,"invalid value (%d in field %d)",value,field);
 	dsc->error = 1;
     }
 }
@@ -247,21 +247,21 @@ void q_assign(Q_DSC *dsc,int field,unsigned long value)
 void q_write(Q_DSC *dsc,int field,const void *buf,int size)
 {
     if (field < 0 || field >= Q_FIELDS)
-	q_report(Q_FATAL,"invalid field value (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid field value (%d)",field);
     if (fields[field].pos & 7)
-	q_report(Q_FATAL,"invalid use of q_write (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid use of q_write (%d)",field);
     if (fields[field].actual >= 0) {
 	if (size > fields[field].size/8) {
-	    q_report(Q_ERROR,"%d bytes too big for %d byte field %d",size,
-	      fields[field].size/8,field);
+	    PREFIX(report)(Q_ERROR,"%d bytes too big for %d byte field %d",
+	      size,fields[field].size/8,field);
 	    dsc->error = 1;
 	    return;
 	}
 	dsc->length[fields[field].actual] = size;
     }
     else if ((fields[field].pos | fields[field].size) & 7)
-	    q_report(Q_FATAL,"field %d is neither var-len nor well-shaped",
-	      field);
+	    PREFIX(report)(Q_FATAL,"field %d is neither var-len nor "
+	      "well-shaped",field);
     memcpy(dsc->data+(fields[field].pos/8),buf,(size_t) size);
     q_set(dsc->field_present,field);
     use_group(dsc,fields[field].parent);
@@ -272,14 +272,15 @@ int q_present(const Q_DSC *dsc,int field)
 {
     if (field < 0) {
 	if (field < -Q_GROUPS)
-	    q_report(Q_FATAL,"invalid group number (%d)",field);
-	return q_test(dsc->group_present,-field-1);
+	    PREFIX(report)(Q_FATAL,"invalid group number (%d)",field);
+	return q_test((unsigned char *) dsc->group_present,-field-1);
     }
     else {
 	if (field >= Q_FIELDS)
-	    q_report(Q_FATAL,"invalid field number (%d)",field);
+	    PREFIX(report)(Q_FATAL,"invalid field number (%d)",field);
 	if (q_test(dsc->field_present,field)) return 1;
-	return q_test(dsc->group_present,fields[field].parent);
+	return q_test((unsigned char *) dsc->group_present,
+	  fields[field].parent);
     }
 }
 
@@ -287,7 +288,7 @@ int q_present(const Q_DSC *dsc,int field)
 unsigned long q_fetch(const Q_DSC *dsc,int field)
 {
     if (field < 0 || field >= Q_FIELDS)
-	q_report(Q_FATAL,"invalid field value (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid field value (%d)",field);
     return q_get(dsc->data,fields[field].pos,fields[field].size);
 }
 
@@ -295,11 +296,11 @@ unsigned long q_fetch(const Q_DSC *dsc,int field)
 int q_length(const Q_DSC *dsc,int field)
 {
     if (field < 0 || field >= Q_FIELDS)
-	q_report(Q_FATAL,"invalid field value (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid field value (%d)",field);
     if (fields[field].pos & 7)
-	q_report(Q_FATAL,"invalid use of q_length (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid use of q_length (%d)",field);
     if (fields[field].actual < 0)
-	q_report(Q_FATAL,"field %d is not var-len",field);
+	PREFIX(report)(Q_FATAL,"field %d is not var-len",field);
     return dsc->length[fields[field].actual];
 }
 
@@ -309,16 +310,16 @@ int q_read(Q_DSC *dsc,int field,void *buf,int size)
     int len;
 
     if (field < 0 || field >= Q_FIELDS)
-	q_report(Q_FATAL,"invalid field value (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid field value (%d)",field);
     if (fields[field].pos & 7)
-	q_report(Q_FATAL,"invalid use of q_read (%d)",field);
+	PREFIX(report)(Q_FATAL,"invalid use of q_read (%d)",field);
     if (fields[field].actual >= 0) len = dsc->length[fields[field].actual];
     else if (!(fields[field].size & 7)) len = fields[field].size >> 3;
-	else q_report(Q_FATAL,"field %d is not byte-sized (%d bits)",field,
-	      fields[field].size);
+	else PREFIX(report)(Q_FATAL,"field %d is not byte-sized (%d bits)",
+	      field,fields[field].size);
     if (size < len) {
-	q_report(Q_ERROR,"%d bytes too big for %d byte buffer (field %d)",len,
-	  size,field);
+	PREFIX(report)(Q_ERROR,"%d bytes too big for %d byte buffer (field "
+	  "%d)",len,size,field);
 	dsc->error = 1;
 	return -1;
     }
@@ -336,7 +337,7 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 
     for (i = 0; i < Q_FIELDS; i++)
 	if (q_test(dsc->required,i) && !q_test(dsc->field_present,i))
-	    q_report(Q_ERROR,"required field %d is missing",i);
+	    PREFIX(report)(Q_ERROR,"required field %d is missing",i);
     memset(buf,0,(size_t) size);
     if (q_dump)
 	for (j = 0; j < 100; j += 20) {
@@ -348,15 +349,16 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
     pc = construct; 
     sp = 0;
     while (1) {
-	q_report(Q_DEBUG,"%d(%d):",pc-construct,pos-buf);
+	PREFIX(report)(Q_DEBUG,"%d(%d):",pc-construct,pos-buf);
 	switch (*pc++) {
 	    case OP_COPY:
 		if (size < *pc) {
-		    q_report(Q_ERROR,"not enough space (%d < %d)",size,*pc);
+		    PREFIX(report)(Q_ERROR,"not enough space (%d < %d)",size,
+		      *pc);
 		    dsc->error = 1;
 		    return -1;
 		}
-		q_report(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
+		PREFIX(report)(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
 		q_copy(dsc->data,pc[1],pos,pc[2]);
 		if (q_dump) {
 		    for (i = 0; i < 50; i++) fprintf(stderr,"%02X ",buf[i]);
@@ -368,7 +370,7 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		break;
 	    case OP_COPYVAR:
 		if (size < dsc->length[*pc]) {
-		    q_report(Q_ERROR,"not enough space (%d < %d)",size,
+		    PREFIX(report)(Q_ERROR,"not enough space (%d < %d)",size,
 		      dsc->length[*pc]);
 		    dsc->error = 1;
 		    return -1;
@@ -380,12 +382,13 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		break;
 	    case OP_BEGIN_LEN:
 		if (size < *pc) {
-		    q_report(Q_ERROR,"not enough space (%d < %d)",size,*pc);
+		    PREFIX(report)(Q_ERROR,"not enough space (%d < %d)",size,
+		      *pc);
 		    dsc->error = 1;
 		    return -1;
 		}
 		if (sp == LENGTH_STACK) {
-		    q_report(Q_ERROR,"length stack overflow");
+		    PREFIX(report)(Q_ERROR,"length stack overflow");
 		    dsc->error = 1;
 		    return -1;
 		}
@@ -398,12 +401,12 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		pc += 3;
 		break;
 	    case OP_END_LEN:
-		if (!sp--) q_report(Q_FATAL,"length stack underflow");
+		if (!sp--) PREFIX(report)(Q_FATAL,"length stack underflow");
 		q_put(stack[sp].start,0,stack[sp].size,
 		  (size_t) ((pos-stack[sp].start)-((stack[sp].size+7) >> 3)));
 		break;
 	    case OP_IFGROUP:
-		if (q_test(dsc->group_present,*pc++)) pc++;
+		if (q_test((unsigned char *) dsc->group_present,*pc++)) pc++;
 		else pc += *pc+1;
 		break;
 #if 0
@@ -412,13 +415,15 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		    int len;
 
 		    for (len = *pc++; len; len--)
-			if (!q_test(dsc->group_present,*pc++)) pc++;
+			if (!q_test((unsigned char *) dsc->group_present,*pc++))
+			    pc++;
 			else {
 			    pc += *pc+1;
 			    break;
 			}
 		    if (!len)
-			q_report(Q_FATAL,"multi failed (pc %d)",pc-construct);
+			PREFIX(report)(Q_FATAL,"multi failed (pc %d)",
+			  pc-construct);
 		}
 		break;
 #endif
@@ -428,7 +433,7 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 	    case OP_END:
 		return pos-buf;
 	    default:
-		q_report(Q_FATAL,"unrecognized opcode %d",pc[-1]);
+		PREFIX(report)(Q_FATAL,"unrecognized opcode %d",pc[-1]);
 		return -1; /* for gcc */
 	}
     }
@@ -451,9 +456,9 @@ static void handle_error(Q_DSC *dsc,int size,unsigned char *buf,
 {
     Q_ERR_DSC *error,**last;
 
-    q_report(Q_ERROR,q_err_msg[type],value);
-    q_report(Q_ERROR,"[ PC=%d SP=%d RP=%d, pos=%d end=%d ]",*pc-parse,*sp,*rp,
-      *pos-buf,*end-buf);
+    PREFIX(report)(Q_ERROR,q_err_msg[type],value);
+    PREFIX(report)(Q_ERROR,"[ PC=%d SP=%d RP=%d, pos=%d end=%d ]",*pc-parse,
+      *sp,*rp,*pos-buf,*end-buf);
     error = alloc_t(Q_ERR_DSC);
     error->type = type;
     error->pc = *pc-parse;
@@ -501,7 +506,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
     pc = parse; 
     sp = rp = 0;
     while (1) {
-	q_report(Q_DEBUG,"%d(%d):",pc-parse,pos-buf);
+	PREFIX(report)(Q_DEBUG,"%d(%d):",pc-parse,pos-buf);
 	switch (*pc++) {
 #ifdef DUMP_MODE
 	    case OP_DUMP:
@@ -536,10 +541,10 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 #endif
 	    case OP_COPY:
 		if (pos+*pc > end) {
-		    ERROR(qet_space,end-buf);
+		    ERROR(qet_space,end-pos);
 		    continue;
 		}
-		q_report(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
+		PREFIX(report)(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
 		q_copy(pos,pc[1] & 7,dsc->data+(pc[1] >> 3),pc[2]);
 		if (q_dump) {
 		    for (i = 0; i < 20; i++)
@@ -556,7 +561,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		    len = end-pos;
 		    if (len > pc[2]) len = pc[2];
 		    memcpy(dsc->data+pc[1]/8,pos,(size_t) len);
-		    q_report(Q_DEBUG,"len %d for %d",len,*pc);
+		    PREFIX(report)(Q_DEBUG,"len %d for %d",len,*pc);
 		    dsc->length[*pc] = len;
 		    pos += len;
 		    pc += 3;
@@ -564,23 +569,24 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		}
 	    case OP_BEGIN_LEN:
 		if (pos+*pc > end) {
-		    ERROR(qet_space,end-buf);
+		    ERROR(qet_space,end-pos);
 		    continue;
 		}
 		if (sp == LENGTH_STACK)
-		    q_report(Q_FATAL,"length stack overflow");
+		    PREFIX(report)(Q_FATAL,"length stack overflow");
 		stack[sp] = end;
 		end = pos+q_get(pos,pc[1] & 7,pc[2])+*pc;
-		if (end > stack[sp]) q_report(Q_FATAL,"length has grown");
+		if (end > stack[sp])
+		    PREFIX(report)(Q_FATAL,"length has grown");
 		pos += *pc;
 		sp++;
 		pc += 3;
 		break;
 	    case OP_BEGIN_REC:
-		q_report(Q_DEBUG,"begin_rec pc %d sp %d pos %d end %d",
+		PREFIX(report)(Q_DEBUG,"begin_rec pc %d sp %d pos %d end %d",
 		  pc-parse,sp,pos-buf,end-buf);
 		if (rp == LENGTH_R_STACK)
-		    q_report(Q_FATAL,"recovery stack overflow");
+		    PREFIX(report)(Q_FATAL,"recovery stack overflow");
 		r_stack[rp].pc = pc;
 		r_stack[rp].sp = sp;
 		r_stack[rp].pos = pos;
@@ -589,19 +595,19 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		pc += 3;
 		break;
 	    case OP_END_LEN:
-		if (!sp--) q_report(Q_FATAL,"length stack underflow");
+		if (!sp--) PREFIX(report)(Q_FATAL,"length stack underflow");
 		end = stack[sp];
 		break;
 	    case OP_END_REC:
-		q_report(Q_DEBUG,"end_rec");
-		if (!rp--) q_report(Q_FATAL,"recovery stack underflow");
+		PREFIX(report)(Q_DEBUG,"end_rec");
+		if (!rp--) PREFIX(report)(Q_FATAL,"recovery stack underflow");
 		break;
 	    case OP_CASE:
 		{
 		    int len,value,group;
 
 		    if (pos+*pc > end) {
-			ERROR(qet_space,end-buf);
+			ERROR(qet_space,end-pos);
 			continue;
 		    }
 		    value = q_get(pos,pc[1] & 7,pc[2]);
@@ -613,7 +619,8 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 			    pc++;
 			    for (group = *pc++; group != -1; group =
 			      groups[group].parent)
-				q_set(dsc->group_present,group);
+				q_set((unsigned char *) dsc->group_present,
+				  group);
 			    pc += *pc+1;
 			    break;
 			}
@@ -627,7 +634,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		pc += *pc+1;
 		break;
 	    case OP_IFEND:
-		q_report(Q_DEBUG,"ifend - %d/%d",pos-buf,end-buf);
+		PREFIX(report)(Q_DEBUG,"ifend - %d/%d",pos-buf,end-buf);
 		if (pos == end) pc += *pc;
 		pc++;
 		break;
@@ -637,7 +644,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 	    case OP_END:
 		return dsc->errors ? -1 : 0;
 	    default:
-		q_report(Q_FATAL,"unrecognized opcode %d",pc[-1]);
+		PREFIX(report)(Q_FATAL,"unrecognized opcode %d",pc[-1]);
 		return -1; /* for gcc */
 	}
     }
@@ -655,13 +662,13 @@ static void fixups(Q_DSC *dsc)
 
     for (walk = dsc->errors; walk; walk = walk->next) {
 	gs[gp = 0] = walk->group;
-	q_clear(dsc->group_present,walk->group);
+	q_clear((unsigned char *) dsc->group_present,walk->group);
 	for (i = walk->group+1; i < Q_GROUPS; i++) {
 	    while (groups[i].parent != gs[gp])
 		if (!gp--) break;
 	    if (gp < 0) break;
 	    gs[++gp] = i;
-	    q_clear(dsc->group_present,i);
+	    q_clear((unsigned char *) dsc->group_present,i);
 	}
 	for (j = 0; j < Q_FIELDS; j++)
 	     if (fields[j].parent == walk->group) break;
@@ -698,7 +705,7 @@ static int q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 	    putc('\n',stderr);
 	}
 	for (i = 0; i < Q_GROUPS; i++)
-	    if (q_test(dsc->group_present,i))
+	    if (q_test((unsigned char *) dsc->group_present,i))
 		for (p = groups[i].required; p && *p != -1; p++)
 		    fprintf(stderr,"%d: %ld / 0x%lx\n",*p,
 		      q_get(dsc->data,fields[*p].pos,fields[*p].size),
