@@ -13,7 +13,11 @@
 #endif
 
 
+#ifdef UNI40
+#include "q40.out.h"
+#else
 #include "q.out.h"
+#endif
 
 
 /*
