@@ -126,13 +126,21 @@ static int open_signaling(void)
 	return -1;
     }
     memset(&qos,0,sizeof(qos));
-    if (sig_pcr == -1)
-	qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_UBR;
-    else {
-	qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_CBR;
-	qos.rxtp.min_pcr = qos.txtp.min_pcr = sig_pcr;
-    }
     qos.rxtp.max_sdu = qos.txtp.max_sdu = MAX_Q_MSG;
+    if (sig_qos) {
+	if (text2qos(sig_qos,&qos,T2Q_DEFAULTS) < 0) {
+	    fprintf(stderr,"invalid qos: %s\n",sig_qos);
+	    return -1;
+	}
+    }
+    else {
+	if (sig_pcr == -1)
+	    qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_UBR;
+	else {
+	    qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_CBR;
+	    qos.rxtp.min_pcr = qos.txtp.min_pcr = sig_pcr;
+	}
+    }
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
 	perror("setsockopt SO_ATMQOS");
 	return -1;
