@@ -46,8 +46,8 @@ int main(int argc,char **argv)
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),
       T2A_PVC | T2A_UNSPEC | T2A_WILDCARD) < 0) usage(argv[0]);
     memset(&qos,0,sizeof(qos));
-    qos.txtp.class = ATM_UBR;
-    qos.txtp.class = ATM_UBR;
+    qos.txtp.traffic_class = ATM_UBR;
+    qos.txtp.traffic_class = ATM_UBR;
     qos.txtp.max_sdu = 1;
     qos.rxtp = qos.txtp;
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
